@@ -56,6 +56,7 @@ goog.require('goog.math.Vec2');
  * @defines // INVALID_JSDOC_TAG
  * @nginject // INVALID_JSDOC_TAG
  * @wizAction // INVALID_JSDOC_TAG
+ * @polymerbehavior // INVALID_JSDOC_TAG
  */
 function badTags() {
 }
@@ -90,6 +91,18 @@ function good(object) {
 }
 
 
+/**
+ * Some documentation goes here.
+ *
+ * @polymerBehavior
+ */
+var goodPolymerBehavior = {
+  properties: {
+    foo: String,
+  },
+};
+
+
 /**
  * Some documentation goes here.
  * @param {function(string, string) : string} f A function.
@@ -142,6 +155,9 @@ function missingParamButInherit(object) {
  * @param {Object} object Docs.
  */
 function mismatchedParam(elem) { // WRONG_PARAMETER_DOCUMENTATION
+  /** @param {number} otherElem */
+  function nestedFunction(elem) { // WRONG_PARAMETER_DOCUMENTATION
+  };
 }
 
 
@@ -208,6 +224,15 @@ function unnecessaryMissingReturnDoc() {
 }
 
 
+/**
+ * The "suppress" causes the compiler to ignore the 'debugger' statement.
+ * @suppress {checkDebuggerStatement}
+ */
+function checkDebuggerStatementWithSuppress() {
+  debugger;
+}
+
+
 /**
  * Return doc is present, but the function doesn't have a 'return' statement.
  * The "suppress" causes the compiler to ignore the error.
@@ -306,6 +331,24 @@ function OkNoReturnWithConstructor() {
 }
 
 
+/**
+ * Type of array is known, so the cast is unnecessary.
+ * @suppress {unnecessaryCasts}
+ */
+function unnecessaryCastWithSuppress() {
+  var numberArray = /** @type {!Array.<number>} */ ([]);
+  /** @type {number} */ (goog.array.peek(numberArray));
+}
+
+
+
+/**
+ * Make sure the 'unrestricted' annotation is accepted.
+ * @constructor @unrestricted
+ */
+function UnrestrictedClass() {}
+
+
 
 /**
  * Check definition of fields in constructors.
@@ -465,6 +508,22 @@ class.__iterator__ = function() {
 };
 
 
+/**
+ * Some docs.
+ * @package
+ */
+class.goodPackage = function() {
+};
+
+
+/**
+ * Some docs.
+ * @package
+ */
+class.badPackage_ = function() { // MISSING_PRIVATE
+};
+
+
 /**
  * Some docs.
  * @protected
@@ -581,6 +640,45 @@ class.goodReturn = function() {
 };
 
 
+/** @type {Array.<Object|null>} // JSDOC_PREFER_QUESTION_TO_PIPE_NULL */
+class.badType;
+
+
+/**
+ * For template types, the ?TYPE notation is not parsed correctly by the
+ * compiler, so don't warn here.
+ * @type {Array.<TYPE|null>}
+ * @template TYPE
+ */
+class.goodTemplateType;
+
+
+// As the syntax may look ambivalent: The function returns just null.
+/** @type {function():null|Object} */
+class.goodType;
+
+
+/** @type {function():(null|Object)} // JSDOC_PREFER_QUESTION_TO_PIPE_NULL */
+class.badType;
+
+
+// As the syntax may look ambivalent: The function returns just Object.
+/** @type {function():Object|null} // JSDOC_PREFER_QUESTION_TO_PIPE_NULL */
+class.badType;
+
+
+/** @type {(function():Object)|null} // JSDOC_PREFER_QUESTION_TO_PIPE_NULL */
+class.badType;
+
+
+/** @type {function(null,Object)} */
+class.goodType;
+
+
+/** @type {{a:null,b:Object}} */
+class.goodType;
+
+
 // +2: JSDOC_PREFER_QUESTION_TO_PIPE_NULL
 /**
  * @return {Object|null} A bad return.
@@ -820,7 +918,7 @@ class.goodOverrideDocs = function() {
  * Test that flags embedded in docs don't trigger ends with invalid character
  * error.
  * @bug 2983692
- * @deprecated Please use the {@code @typedef} annotation.
+ * @deprecated Please use the {@code @hidden} annotation.
  */
 function goodEndChar() {
 }
@@ -899,7 +997,7 @@ bar.foo = null;
 
 /**
  * @extends Object} //MISSING_BRACES_AROUND_TYPE
- */
+ */ // JSDOC_DOES_NOT_PARSE
 bar.baz = x;
 
 
@@ -1030,8 +1128,8 @@ test.x.y = function() {
  * parsed as being here.
  * @param {Event} e The event.
  */
-detroit.commands.ChangeOwnerCommand.
-    prototype.handleDocumentStoreCompleteEvent = function(e) {
+detroit.commands.ChangeOwnerCommand
+    .prototype.handleDocumentStoreCompleteEvent = function(e) {
   this.x = e.target;
 };
 
@@ -1215,18 +1313,18 @@ TypelessEnumTest = {
 // prototype.
 x.prototype = {};
 
-y.
-    prototype = {};
+y
+    .prototype = {};
 
-x.y.
-    z.prototype = {};
+x.y
+    .z.prototype = {};
 
 x.myprototype = {}; // MISSING_MEMBER_DOCUMENTATION
 
 x.prototype.y = 5; // MISSING_MEMBER_DOCUMENTATION
 
-x.prototype.
-    y.z = {}; // MISSING_MEMBER_DOCUMENTATION
+x.prototype
+    .y.z = {}; // MISSING_MEMBER_DOCUMENTATION
 
 
 /** @typedef {(string|number)} */
@@ -1342,6 +1440,31 @@ Foo.prototype = {
   }
 };
 
+/** Regression tests for annotation types with spaces. */
+
+
+/** @enum {goog.events.Event<string, number>} */
+var Bar;
+
+
+/** @export {goog.events.Event<string, number>} */
+var Bar;
+
+
+/** @package {goog.events.Event<string, number>} */
+var Bar;
+
+
+
+/**
+ * @constructor
+ * @implements {goog.dom.Range<string, number>}
+ */
+var Foo = function() {
+  /** @final {goog.events.Event<string, number>} */
+  this.bar = null;
+};
+
 /* Regression tests for not ending block comments. Keep at end of file! **/
 /**
  * When there are multiple asteriks. In the failure case we would get an
